﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LazyEnumerableEvaluation {

	class DataSource : IEnumerable<int> {
		public int From { get; private set; }
		public int To { get; private set; }

		public DataSource(int from, int to) {
			From = from;
			To = to;
		}

		public IEnumerator<int> GetEnumerator() {
			for (int i = From; i <= To; i++) {
				Console.WriteLine($"DataSource yielding value {i}");
				yield return i;
			}
		}

		IEnumerator IEnumerable.GetEnumerator() {
			return GetEnumerator();
		}
	}

	class Program {
		static void PrintFirst3(List<int> source) {
			for (int i = 0; i < 3; i++) { 
				Console.WriteLine($"List<int>: Value #{i + 1}: {source[i]}");
			}
		}

		static void Main(string[] args) {
			var data = new DataSource(1, 20);

			PrintFirst3(new List<int>(data));

			Console.WriteLine();

			PrintFirst3(data);

			Console.WriteLine();

			Test2();
		}

		static void PrintFirst3(IEnumerable<int> source) {
			int i = 1;
			foreach (var value in source) {
				Console.WriteLine($"IE<int>: Value #{i}: {value}");
				if (i++ == 3) break;
			}
		}

		static void Test2() {
			var random = new RandomSource(10);

			PrintFirst3(random);

			Console.WriteLine();

			// Will run forever!
			// PrintFirst3(new List<int>(random));
		}
	}

	class RandomSource : IEnumerable<int> {
		private readonly Random random = new Random();

		public int To { get; private set; }

		public RandomSource(int to) {
			To = to;
		}

		public IEnumerator<int> GetEnumerator() {
			while (true) {
				Console.WriteLine("RandomSource yielding next value ...");
				yield return random.Next(To);
			}
		}

		IEnumerator IEnumerable.GetEnumerator() {
			return GetEnumerator();
		}
	}
}
